#include "stdafx.h"
#include "WinNotify.h"
#include "WinNotifyDoc.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#pragma warning(suppress: 26409 26433 26440 26477)
IMPLEMENT_DYNCREATE(CWinNotifyDoc, CDocument)

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CWinNotifyDoc, CDocument) //NOLINT(clang-diagnostic-unused-local-typedef, modernize-avoid-c-arrays)
  ON_COMMAND(ID_MESSAGE_CLEARALL, &CWinNotifyDoc::OnMessageClearAll)
  ON_UPDATE_COMMAND_UI(ID_MESSAGE_CLEARALL, &CWinNotifyDoc::OnUpdateMessageClearAll)
  ON_COMMAND(ID_MESSAGE_DISCARD, &CWinNotifyDoc::OnMessageDiscard)
  ON_UPDATE_COMMAND_UI(ID_MESSAGE_DISCARD, &CWinNotifyDoc::OnUpdateMessageDiscard)
  ON_COMMAND(ID_MESSAGE_NEXT, &CWinNotifyDoc::OnMessageNext)
  ON_UPDATE_COMMAND_UI(ID_MESSAGE_NEXT, &CWinNotifyDoc::OnUpdateMessageNext)
  ON_COMMAND(ID_MESSAGE_PREVIOUS, &CWinNotifyDoc::OnMessagePrevious)
  ON_UPDATE_COMMAND_UI(ID_MESSAGE_PREVIOUS, &CWinNotifyDoc::OnUpdateMessagePrevious)
  ON_COMMAND(ID_MESSAGE_SEND, &CWinNotifyDoc::OnMessageSend)
END_MESSAGE_MAP()


CWinNotifyDoc::CWinNotifyDoc() noexcept : m_nCurrentMessageIndex{0}
{
}

BOOL CWinNotifyDoc::OnNewDocument()
{
  //Let the base class do its thing
  if (!__super::OnNewDocument())
    return FALSE;

  //Create the server mailslot to listen for incoming messages
  if (!m_ServerMailslot.Open(_T("\\\\.\\mailslot\\WinNot\\Server")))
  {
    const DWORD dwError{GetLastError()};
    LPTSTR lpBuffer{nullptr};
#pragma warning(suppress: 26490)
    const DWORD dwReturn{FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                                       nullptr,  dwError, MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
                                       reinterpret_cast<LPTSTR>(&lpBuffer), 0, nullptr)};

    CString sError;
    if (dwReturn != 0)
    {
      sError.Format(_T("%u, %s"), dwError, lpBuffer);
      LocalFree(lpBuffer);
    }
    else
      sError.Format(_T("%u"), dwError);

    CString sMessage;
    AfxFormatString1(sMessage, IDP_FAIL_CREATE_SERVER_MAILSLOT, sError);
    AfxMessageBox(sMessage, MB_OK | MB_ICONSTOP);
    return FALSE;
  }

  return TRUE;
}

INT_PTR CWinNotifyDoc::GetCurrentMessageCount() const
{
  //Return the current number of messages
  if (m_Messages.GetSize())
    return m_nCurrentMessageIndex+1;
  else
    return 0;
}

void CWinNotifyDoc::Serialize(CArchive& /*ar*/) noexcept
{
  //We do not do anything with the hard disk
}

void CWinNotifyDoc::OnMessageClearAll()
{
  //remove all messages from the array and inform all views
  m_Messages.RemoveAt(0, m_Messages.GetSize());
  m_nCurrentMessageIndex = 0;
  UpdateAllViews(nullptr);
}

#pragma warning(suppress: 26429)
void CWinNotifyDoc::OnUpdateMessageClearAll(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486)
  pCmdUI->Enable(m_Messages.GetSize() > 0);
}

void CWinNotifyDoc::OnMessageDiscard()
{
  //Remove the current message from the array and inform all views
  m_Messages.RemoveAt(m_nCurrentMessageIndex);
  if (m_nCurrentMessageIndex > m_Messages.GetUpperBound())
    m_nCurrentMessageIndex = m_Messages.GetUpperBound();
  UpdateAllViews(nullptr);
}

#pragma warning(suppress: 26429)
void CWinNotifyDoc::OnUpdateMessageDiscard(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486)
  pCmdUI->Enable(m_Messages.GetSize() > 0);
}

void CWinNotifyDoc::OnMessageNext()
{
  //Move to the next message and inform all views
  ++m_nCurrentMessageIndex;
  UpdateAllViews(nullptr);
}

#pragma warning(suppress: 26429)
void CWinNotifyDoc::OnUpdateMessageNext(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486)
  pCmdUI->Enable(m_nCurrentMessageIndex < m_Messages.GetUpperBound());
}

void CWinNotifyDoc::OnMessagePrevious()
{
  //Move to the previous message and inform all views
  --m_nCurrentMessageIndex;
  UpdateAllViews(nullptr);
}

#pragma warning(suppress: 26429)
void CWinNotifyDoc::OnUpdateMessagePrevious(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486)
  pCmdUI->Enable(m_nCurrentMessageIndex > 0);
}

void CWinNotifyDoc::OnMessageSend()
{
  //Send a message by opening a client side mailslot and writing to it
  CSendDlg dlg;
  if (dlg.DoModal() == IDOK)
  {
    //Display a wait cursor
    CWaitCursor wait;

    CClientMailslot mailslot;
    CString sMailSlot;
    sMailSlot.Format(_T("\\\\%s\\mailslot\\WinNot\\Server"), dlg.m_sTo.GetString());
    if (!mailslot.Open(sMailSlot))
    {
      const DWORD dwError{GetLastError()};
      LPTSTR lpBuffer{nullptr};
#pragma warning(suppress: 26490)
      const DWORD dwReturn{FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                                         nullptr,  dwError, MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
                                         reinterpret_cast<LPTSTR>(&lpBuffer), 0, nullptr)};

      CString sError;
      if (dwReturn != 0)
      {
        sError.Format(_T("%u, %s"), dwError, lpBuffer);
        LocalFree(lpBuffer);
      }
      else
        sError.Format(_T("%u"), dwError);

      CString sMessage;
      AfxFormatString1(sMessage, IDP_FAIL_CREATE_CLIENT_MAILSLOT, sError);
      AfxMessageBox(sMessage, MB_OK | MB_ICONEXCLAMATION);
      return;
    }

    //create the message
    CNotifyMessage msg;
#pragma warning(suppress: 26485)
    _tcscpy_s(msg.m_sMessage, sizeof(msg.m_sMessage)/sizeof(TCHAR), dlg.m_sMessage);

    CString sUserName;
    DWORD dwUserNameSize{CNotifyMessage::MaxNameSize};
    const BOOL bGetUserName{GetUserName(sUserName.GetBuffer(CNotifyMessage::MaxNameSize), &dwUserNameSize)};
    sUserName.ReleaseBuffer();
    if (bGetUserName)
#pragma warning(suppress: 26485)
      _tcscpy_s(msg.m_sSenderName, _countof(msg.m_sSenderName), sUserName);
    else
#pragma warning(suppress: 26485)
      _tcscpy_s(msg.m_sSenderName, _countof(msg.m_sSenderName), _T("Unknown"));

    CString sComputerName;
    DWORD dwComputerNameSize{CNotifyMessage::MaxNameSize};
    const BOOL bGetComputerName{GetComputerName(sComputerName.GetBuffer(CNotifyMessage::MaxNameSize), &dwComputerNameSize)};
    sComputerName.ReleaseBuffer();
    if (bGetComputerName)
#pragma warning(suppress: 26485)
      _tcscpy_s(msg.m_sSenderMachine, _countof(msg.m_sSenderMachine), sComputerName);
    else
#pragma warning(suppress: 26485)
      _tcscpy_s(msg.m_sSenderMachine, _countof(msg.m_sSenderMachine), _T("Unknown"));

#pragma warning(suppress: 26485)
    _tcscpy_s(msg.m_sTo, _countof(msg.m_sTo), dlg.m_sTo);
    GetLocalTime(&msg.m_SentTime);

    //Send the message
    DWORD dwBytesWritten{0};
    constexpr const DWORD dwBytesToWrite{sizeof(CNotifyMessage)};
    if (!mailslot.Write(&msg, dwBytesToWrite, &dwBytesWritten) || (dwBytesToWrite != dwBytesWritten))
    {
      //Display the error
      const DWORD dwError{GetLastError()};
      LPTSTR lpBuffer{nullptr};
#pragma warning(suppress: 26490)
      const DWORD dwReturn{FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                                         nullptr, dwError, MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
                                         reinterpret_cast<LPTSTR>(&lpBuffer), 0, nullptr)};

      CString sError;
      if (dwReturn != 0)
      {
        sError.Format(_T("%u, %s"), dwError, lpBuffer);
        LocalFree(lpBuffer);
      }
      else
        sError.Format(_T("%u"), dwError);
    
      CString sMsg;
      AfxFormatString1(sMsg, IDP_FAIL_SEND_MESSAGE, sError);
      AfxMessageBox(sMsg, MB_OK | MB_ICONEXCLAMATION);
    }
  }
}

void CWinNotifyDoc::CheckForNewMail()
{
  if (m_ServerMailslot.IsOpen())
  {
    //Read all the messages waiting in the server mailslot
    DWORD dwMessageCount{0};
    DWORD dwMsgSize{0};
    while (m_ServerMailslot.GetInfo(nullptr, &dwMsgSize, &dwMessageCount, nullptr) && dwMessageCount)
    {
      //Read the message from the mailslot
      std::vector<BYTE> msgData(dwMsgSize);
      DWORD dwNumberOfBytesRead{0};
      if (m_ServerMailslot.Read(msgData.data(), dwMsgSize, &dwNumberOfBytesRead))
      {
        //Add it to the array if a valid size
        if (dwNumberOfBytesRead == sizeof(CNotifyMessage))
        {
          CNotifyMessage msg;
          memcpy(&msg, msgData.data(), dwNumberOfBytesRead);
          m_Messages.Add(msg);
          m_nCurrentMessageIndex = m_Messages.GetUpperBound();
          UpdateAllViews(nullptr);
          MessageBeep(MB_ICONEXCLAMATION);
        }
      }
    }
  }
}

CString CWinNotifyDoc::GetCurrentMessage() const
{
  //What will be return value from this function
  CString sText;

  if (m_Messages.GetSize())
#pragma warning(suppress: 26485)
    sText = m_Messages.GetAt(m_nCurrentMessageIndex).m_sMessage;

  return sText;
}

CString CWinNotifyDoc::GetCurrentSender() const
{
  //What will be the return value from this function
  CString sText;

  if (m_Messages.GetSize())
#pragma warning(suppress: 26485)
    sText = m_Messages.GetAt(m_nCurrentMessageIndex).m_sSenderName;

  return sText;
}

CString CWinNotifyDoc::GetCurrentSenderMachine() const
{
  //What will be the return value from this function
  CString sText;

  if (m_Messages.GetSize())
#pragma warning(suppress: 26485)
    sText = m_Messages.GetAt(m_nCurrentMessageIndex).m_sSenderMachine;

  return sText;
}

CString CWinNotifyDoc::GetCurrentTo() const
{
  //What will be the return value from this function
  CString sText;

  if (m_Messages.GetSize())
#pragma warning(suppress: 26485)
    sText = m_Messages.GetAt(m_nCurrentMessageIndex).m_sTo;

  return sText;
}


CString CWinNotifyDoc::GetCurrentMessageTime()
{
  //What will be the return value from this function
  CString sText;

  if (m_Messages.GetSize())
  {
    CString sDateStr;
    CString sTimeStr;
    const CNotifyMessage& curMessage{m_Messages.ElementAt(m_nCurrentMessageIndex)};
    GetDateFormat(LOCALE_USER_DEFAULT, 0, &curMessage.m_SentTime, nullptr, sDateStr.GetBuffer(64), 64);
    sDateStr.ReleaseBuffer();
    GetTimeFormat(LOCALE_USER_DEFAULT, 0, &curMessage.m_SentTime, nullptr, sTimeStr.GetBuffer(64), 64);
    sTimeStr.ReleaseBuffer();
    sText = sDateStr + _T(" ") + sTimeStr;
  }

  return sText;
}


#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CSendDlg, CDialog) //NOLINT(clang-diagnostic-unused-local-typedef, modernize-avoid-c-arrays)
END_MESSAGE_MAP()

#pragma warning(suppress: 26455)
CSendDlg::CSendDlg(_In_opt_ CWnd* pParent) : CDialog{CSendDlg::IDD, pParent}
{
}

#pragma warning(suppress: 26429)
void CSendDlg::DoDataExchange(CDataExchange* pDX)
{
  __super::DoDataExchange(pDX);
  DDX_Text(pDX, IDC_TO, m_sTo);
  DDX_Text(pDX, IDC_MESSAGE, m_sMessage);
  pDX->PrepareEditCtrl(IDC_TO);
  DDV_MaxChars(pDX, m_sTo, CNotifyMessage::MaxNameSize);
  pDX->PrepareEditCtrl(IDC_MESSAGE);
  DDV_MaxChars(pDX, m_sMessage, CNotifyMessage::MaxMessageSize);
}
