#pragma once


#include "Mailslot.h"


//Structure which will be sent down the mailslot
#pragma pack(push, 1)
class CNotifyMessage
{
public:
//Enums
  enum
  {
    MaxNameSize = MAX_COMPUTERNAME_LENGTH,
    MaxMessageSize = 100
  };

//Member variables
  SYSTEMTIME m_SentTime;
  TCHAR m_sSenderName[MaxNameSize + 1]; //NOLINT(modernize-avoid-c-arrays)
  TCHAR m_sSenderMachine[MaxNameSize + 1]; //NOLINT(modernize-avoid-c-arrays)
  TCHAR m_sTo[MaxNameSize + 1]; //NOLINT(modernize-avoid-c-arrays)
  TCHAR m_sMessage[MaxMessageSize + 1]; //NOLINT(modernize-avoid-c-arrays)
};
#pragma pack(pop)


class CWinNotifyDoc : public CDocument
{
protected: //create from serialization only
  CWinNotifyDoc() noexcept;

public:
//Constructors / Destructors
  CWinNotifyDoc(const CWinNotifyDoc&) = delete;
  CWinNotifyDoc(CWinNotifyDoc&&) = delete;
  ~CWinNotifyDoc() = default; //NOLINT(modernize-use-override)

//Methods
  CWinNotifyDoc& operator=(const CWinNotifyDoc&) = delete;
  CWinNotifyDoc& operator=(CWinNotifyDoc&&) = delete;
  BOOL OnNewDocument() override;
  void Serialize(CArchive& ar) noexcept override;
  void CheckForNewMail();
  _NODISCARD CString GetCurrentMessage() const;
  _NODISCARD CString GetCurrentSender() const;
  _NODISCARD CString GetCurrentSenderMachine() const;
  _NODISCARD CString GetCurrentTo() const;
  _NODISCARD CString GetCurrentMessageTime();
  _NODISCARD INT_PTR GetNumberMessages() const { return m_Messages.GetSize(); };
  _NODISCARD INT_PTR GetCurrentMessageCount() const;

protected:
//Message handlers
  afx_msg void OnMessageClearAll();
  afx_msg void OnUpdateMessageClearAll(CCmdUI* pCmdUI);
  afx_msg void OnMessageDiscard();
  afx_msg void OnUpdateMessageDiscard(CCmdUI* pCmdUI);
  afx_msg void OnMessageNext();
  afx_msg void OnUpdateMessageNext(CCmdUI* pCmdUI);
  afx_msg void OnMessagePrevious();
  afx_msg void OnUpdateMessagePrevious(CCmdUI* pCmdUI);
  afx_msg void OnMessageSend();

//Member variables
  CArray<CNotifyMessage, CNotifyMessage&> m_Messages;
  INT_PTR m_nCurrentMessageIndex;
  CServerMailslot m_ServerMailslot;

  DECLARE_MESSAGE_MAP()
  DECLARE_DYNCREATE(CWinNotifyDoc)
};


class CSendDlg : public CDialog
{
public:
//Constructors / Destructors
  CSendDlg(_In_opt_ CWnd* pParent = nullptr);
  CSendDlg(const CSendDlg&) = delete;
  CSendDlg(CSendDlg&&) = delete;
  ~CSendDlg() = default; //NOLINT(modernize-use-override)

//Methods
  CSendDlg& operator=(const CSendDlg&) = delete;
  CSendDlg& operator=(CSendDlg&&) = delete;

//Member variables
  enum { IDD = IDD_SEND };
  CString m_sMessage;
  CString m_sTo;

protected:
//Methods
  void DoDataExchange(CDataExchange* pDX) override;

  DECLARE_MESSAGE_MAP()
};
